package ch.std.jumpstart.json;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.std.jumpstart.tools.StringUtil;

public class JSONServiceBase {

    private static final Logger logger = LoggerFactory.getLogger(JSONServiceBase.class);

    protected Map<String, Object> rootMap;
    
    public JSONServiceBase() {
    	this.rootMap = new HashMap<>();
    	this.rootMap.put("head", new HashMap<String, Object>());
    	this.rootMap.put("body", new HashMap<String, Object>());
    }

    @SuppressWarnings("unchecked")
	public JSONServiceBase(Object obj) {    	
    	 if (obj instanceof Map<?,?>) {
    		 this.rootMap = (Map<String,Object>)obj;
         } else {
         	this.rootMap = new HashMap<>();
        	this.rootMap.put("head", new HashMap<String, Object>());
        	this.rootMap.put("body", new HashMap<String, Object>());        	 
         }
    }

    @SuppressWarnings("unchecked")
	public Map<String, Object> getHead() {
    	return (Map<String, Object>)this.rootMap.get("head");
    }

    @SuppressWarnings("unchecked")
    public Map<String, Object> getBody() {
    	return (Map<String, Object>)this.rootMap.get("body");
    }
    
    public Object getHeadValue(String key) {
    	return this.getHead().get(key);
    }

    public void setHeadValue(String key, Object value) {
    	this.getHead().put(key,  value);
    }
    
    public Object getBodyValue(String key) { 
    	return this.getBody().get(key);
    }

    @SuppressWarnings("unchecked")
	public Map<String, Object> getBodyMap(String key) { 
    	return (Map<String, Object>)this.getBody().get(key);
    }

    @SuppressWarnings("unchecked")
	public List<Map<String, Object>> getBodyList(String key) { 
    	return (List<Map<String, Object>>)this.getBody().get(key);
    }

    public void setBodyValue(String key, Object value) {
    	this.getBody().put(key,  value);
    }

    @JsonIgnore
    public String getStatus() {
        try {
            return this.getHeadValue("status").toString();
        } catch(Exception e) {
            return null;
        }
    }

    @JsonIgnore
    public String getStatusMessage() {
        try {
            return this.getHeadValue("statusMessage").toString();
        } catch(Exception e) {
            return null;
        }
    }

    @JsonIgnore
    public boolean isSuccess() {
        return "success".equals(this.getStatus());
    }

    public void setSuccess(String message) {
        try {
            this.setHeadValue("status", "success");
            this.setHeadValue("statusMessage", message);
        } catch(Exception e) {
            logger.error(e.getMessage(), e);
        }
    }

    public void setStatusSuccessIfNotSet(String message) {
        if (this.getHeadValue("status") != null) {
            return;
        }
        this.setSuccess(message);
    }

    @JsonIgnore
    public boolean isFailure() {
        return "failure".equals(this.getStatus());
    }

    public void setFailure(String message) {
        try {
            this.setHeadValue("status", "failure");
            this.setHeadValue("statusMessage", message);
        } catch(Exception e) {
            logger.error(e.getMessage(), e);
        }
    }

    public void setFailure(Throwable t) {
        try {
            this.setHeadValue("status", "failure");
            String message = t.getMessage();
            if (StringUtil.isNullOrEmpty(message)) {
                message = t.toString();
            }
            this.setHeadValue("statusMessage", message);
        } catch(Exception e) {
            logger.error(e.getMessage(), e);
        }
    }
    
}

